"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createMakeStyles = void 0;
const react_1 = require("react");
const Object_fromEntries_1 = require("./tools/polyfills/Object.fromEntries");
const objectKeys_1 = require("./tools/objectKeys");
const cssAndCx_1 = require("./cssAndCx");
const getDependencyArrayRef_1 = require("./tools/getDependencyArrayRef");
/**
 * @see {@link https://github.com/garronej/tss-react}
 */
function createMakeStyles(params) {
    const { useTheme } = params;
    const getCounter = (() => {
        let counter = 0;
        return () => counter++;
    })();
    /** returns useStyle. */
    function makeStyles() {
        return function (cssObjectByRuleNameOrGetCssObjectByRuleName) {
            const getCssObjectByRuleName = typeof cssObjectByRuleNameOrGetCssObjectByRuleName ===
                "function"
                ? cssObjectByRuleNameOrGetCssObjectByRuleName
                : () => cssObjectByRuleNameOrGetCssObjectByRuleName;
            const outerCounter = getCounter();
            return function useStyles(params) {
                const theme = useTheme();
                const { cx, css } = (0, cssAndCx_1.useCssAndCx)();
                return (0, react_1.useMemo)(() => {
                    const cssObjectByRuleName = getCssObjectByRuleName(theme, params, (() => {
                        let count = 0;
                        return function createRef() {
                            return `tss-react-ref_${outerCounter}_${count++}`;
                        };
                    })());
                    const classes = (0, Object_fromEntries_1.objectFromEntries)((0, objectKeys_1.objectKeys)(cssObjectByRuleName).map(ruleName => [
                        ruleName,
                        css(cssObjectByRuleName[ruleName]),
                    ]));
                    return {
                        classes,
                        theme,
                        css,
                        cx,
                    };
                }, [cx, css, theme, (0, getDependencyArrayRef_1.getDependencyArrayRef)(params)]);
            };
        };
    }
    function useStyles() {
        const theme = useTheme();
        const { css, cx } = (0, cssAndCx_1.useCssAndCx)();
        return { theme, css, cx };
    }
    return { makeStyles, useStyles };
}
exports.createMakeStyles = createMakeStyles;
